/*
 * Copyright (C) 2013  CZ.NIC, z.s.p.o.
 *
 * This file is part of FRED.
 *
 * FRED is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * FRED is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FRED.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
*  @server_i.h
*  header of domain browser corba wrapper
*  pregenerated by $> omniidl -bcxx -Wba -Wbexample -C./src/corba ~/workspace/enum/idl/idl/DomainBrowser.idl
*/
#ifndef DOMAIN_BROWSER_I_H_
#define DOMAIN_BROWSER_I_H_


#include <memory>
#include "src/corba/DomainBrowser.hh"
#include "src/domain_browser/domain_browser.h"

namespace Registry
{
    namespace DomainBrowser
    {

        class DomainBrowserImpl;//pimpl class

        ///domain browser corba interface
        class Server_i: public POA_Registry::DomainBrowser::Server
        {
        private:
            // do not copy
            const std::auto_ptr<Registry::DomainBrowserImpl::DomainBrowser> pimpl_;

            Server_i(const Server_i&);//no body
            Server_i& operator= (const Server_i&);//no body

        public:
            // standard constructor
            Server_i(const std::string &_server_name,
                    const std::string& _update_registrar_handle,
                    unsigned int domain_list_limit,
                    unsigned int nsset_list_limit,
                    unsigned int keyset_list_limit,
                    unsigned int contact_list_limit);

            virtual ~Server_i();
            // methods corresponding to defined IDL attributes and operations
            Registry::DomainBrowser::TID getObjectRegistryId(
                const char* objtype,
                const char* handle);

            Registry::DomainBrowser::RecordSet* getDomainList(
                const Registry::DomainBrowser::RegistryReference& user_contact,
                const Registry::DomainBrowser::RegistryReference& contact,
                const char* lang,
                ::CORBA::ULong offset,
                ::CORBA::Boolean& limit_exceeded);

            Registry::DomainBrowser::RecordSet* getNssetList(
                const Registry::DomainBrowser::RegistryReference& user_contact,
                const Registry::DomainBrowser::RegistryReference& contact,
                const char* lang,
                ::CORBA::ULong offset,
                 ::CORBA::Boolean& limit_exceeded);

            Registry::DomainBrowser::RecordSet* getKeysetList(
                const Registry::DomainBrowser::RegistryReference& user_contact,
                const Registry::DomainBrowser::RegistryReference& contact,
                const char* lang,
                ::CORBA::ULong offset,
                 ::CORBA::Boolean& limit_exceeded);

            Registry::DomainBrowser::RecordSet* getDomainsForKeyset(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& keyset,
                const char* lang,
                ::CORBA::ULong offset,
                ::CORBA::Boolean& limit_exceeded);

            Registry::DomainBrowser::RecordSet* getDomainsForNsset(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& nsset,
                const char* lang,
                ::CORBA::ULong offset,
                ::CORBA::Boolean& limit_exceeded);

            Registry::DomainBrowser::ContactDetail* getContactDetail(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& detail,
                const char* lang,
                Registry::DomainBrowser::DataAccessLevel& auth_result);

            Registry::DomainBrowser::NSSetDetail* getNssetDetail(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& nsset,
                const char* lang,
                Registry::DomainBrowser::DataAccessLevel& auth_result);

            Registry::DomainBrowser::DomainDetail* getDomainDetail(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& domain,
                const char* lang,
                Registry::DomainBrowser::DataAccessLevel& auth_result);

            Registry::DomainBrowser::KeysetDetail* getKeysetDetail(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::RegistryReference& keyset,
                const char* lang,
                Registry::DomainBrowser::DataAccessLevel& auth_result);

            Registry::DomainBrowser::RegistrarDetail* getRegistrarDetail(
                const Registry::DomainBrowser::RegistryReference& contact,
                const char* handle);

            ::CORBA::Boolean setContactDiscloseFlags(
                const Registry::DomainBrowser::RegistryReference& contact,
                const Registry::DomainBrowser::UpdateContactDiscloseFlags& flags,
                Registry::DomainBrowser::TID request_id);

            ::CORBA::Boolean setAuthInfo(
                const Registry::DomainBrowser::RegistryReference& contact,
                const char* objtype,
                const Registry::DomainBrowser::RegistryReference& objref,
                const char* auth_info,
                Registry::DomainBrowser::TID request_id);

            ::CORBA::Boolean setObjectBlockStatus(const Registry::DomainBrowser::RegistryReference& contact,
                const char* objtype,
                const Registry::DomainBrowser::RegistryReferenceSeq& objects,
                Registry::DomainBrowser::ObjectBlockType block,
                Registry::DomainBrowser::RecordSequence_out blocked);

            Registry::DomainBrowser::RecordSequence* getPublicStatusDesc(const char* lang);

            Registry::DomainBrowser::RecordSet* getMergeContactCandidateList(
                const Registry::DomainBrowser::RegistryReference& contact,
                ::CORBA::ULong offset,
                ::CORBA::Boolean& limit_exceeded);

            void mergeContacts(const Registry::DomainBrowser::RegistryReference& dst_contact,
                const Registry::DomainBrowser::RegistryReferenceSeq& src_contact_list,
                Registry::DomainBrowser::TID request_id);

        };//class Server_i
    }//namespace DomainBrowser
}//namespace Registry

#endif //DOMAIN_BROWSER_I_H_
